<?php
/*
Plugin Name: Geo
Plugin URI: http://redalt.com/wiki/Geo
Description: Attaches geographic coordinate information to posts.
Version: 1.9.1
Author: Owen Winkler
Author URI: http://www.asymptomatic.net
Minimum WordPress Version Required: 2.0
*/

/*
Geo - Attaches geographic coordinate information to posts.
Copyright (c) 2004 Owen Winkler

This program is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any
later version.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 59 Temple Place, Suite 330,
Boston, MA 02111-1307 USA
*/

/*

INSTRUCTIONS:

Drop this file into your WordPress plugins directory, and then
Activate it on the Plugins tab of the WordPress admin console.

See http://dev.wp-plugins.org/wiki/GeoPlugin for more
detailed installation instructions and usage information.

*/


load_plugin_textdomain('Geo');

class Geo
/**
 * Provide a namespace for our plugin
 *
 * @version		1.0
 */
{
	function options_page_geo()
	/**
	* Displays the Options admin interface for the plugin.
	*
	* Long description for the Function.
	*
	* @return	mixed	Description
	*/
	{
		if(isset($_REQUEST['deleteid']))
		{
			$geolocations = get_settings('geo_locations');
			unset($geolocations[$_REQUEST['deleteid']]);
			update_option('geo_locations', $geolocations);
		}
		if(isset($_POST['Options']))
		{
			$use_geo_positions = $_POST['use_geo_positions'] == 1 ? 1 : 0;
			$use_default_geourl = $_POST['use_default_geourl'] == 1 ? 1 : 0;
			update_option('use_geo_positions', $use_geo_positions);
			update_option('use_default_geourl', $use_default_geourl);
			update_option('default_geourl_lat', $_POST['default_geourl_lat']);
			update_option('default_geourl_lon', $_POST['default_geourl_lon']);
			echo '<div class="updated"><p><strong>' . __('Options updated.', 'Geo') . '</strong></p></div>';
	
		}
		if(isset($_POST['Submit']) || isset($_POST['Add']))
		{
	
			$geolocations = get_settings('geo_locations');
			foreach($geolocations as $name => $coords)
			{
				$geolocations[$name] = "{$_POST['lat'][$name]},{$_POST['lon'][$name]}";
			}
			if(isset($_POST['new_location']) && ($_POST['new_location'] != ''))
			{
				$geolocations[$_POST['new_location']] = "{$_POST['new_lat']},{$_POST['new_lon']}";
			}
			update_option('geo_locations', $geolocations);
	
			echo '<div class="updated"><p><strong>' . __('Locations updated.', 'Geo') . '</strong></p></div>';
		}
		else
		{
			Geo::add_options();
		}
	
		$use_geo_positions = get_settings('use_geo_positions');
		$use_default_geourl = get_settings('use_default_geourl');
		$default_geourl_lat = get_settings('default_geourl_lat');
		$default_geourl_lon = get_settings('default_geourl_lon');
	
		$ck_use_geo_positions = $use_geo_positions == 1 ? ' checked="checked"' : '';
		$ck_use_default_geourl[intval($use_default_geourl)] = ' checked="checked"';
	
		$geolocations = get_settings('geo_locations');
		if(!is_array($geolocations)) $geolocations = array();
	
	
		//Page Presentation
		echo '
			<div class="wrap">
			<h2>' . __('Geographic Location Manager', 'Geo') . '</h2>
			<form method="post">
				<table width="100%" cellspacing="2" cellpadding="5" class="editform">
					<tr valign="top">
						<th width="33%" scope="row">' . __('Geographic Tracking Features', 'Geo') . ':</th>
						<td><input type="checkbox" name="use_geo_positions" id="use_geo_positions" ' . $ck_use_geo_positions . ' value="1" /> Enable</td>
					</tr>
					<tr valign="top">
						<th width="33%" scope="row">' . __('When no location is specified', 'Geo') . ':</th>
						<td>
						<label for="use_default_geourl0"><input type="radio" name="use_default_geourl" id="use_default_geourl0" ' . $ck_use_default_geourl[0] . ' value="0" /> ' . __('Do nothing.', 'Geo') . '</label><br />
						<label for="use_default_geourl1"><input type="radio" name="use_default_geourl" id="use_default_geourl1" ' . $ck_use_default_geourl[1] . ' value="1" /> ' . __('Use these:', 'Geo') . '</label>
							<dl>
							<dt><label for="default_geourl_lat">' . __('Latitude', 'Geo') . ':</label></dt>
							<dd><input type="text" name="default_geourl_lat" value="' . $default_geourl_lat . '" /></dd>
							<dt><label for="default_geourl_lon">' . __('Longitude', 'Geo') . ':</label></dt>
							<dd><input type="text" name="default_geourl_lon" value="' . $default_geourl_lon . '" /></dd>
							</dl>
						</td>
					</tr>
				</table>
				<div class="submit"><input type="submit" name="Options" value="' . __('Update Options', 'Geo') . ' &raquo;" /></div>
			</form>
			</div>
		';
	
		echo '
			<div class="wrap">
			<h2>' . __('Preset Locations', 'Geo') . '</h2>
			<form method="post">
			<table width="100%" cellpadding="3" cellspacing="3">
				<thead>
			  <tr>
			    <th scope="col">'.__('Location Name', 'Geo').'</th>
			    <th scope="col">'.__('Latitude', 'Geo').'</th>
			    <th scope="col">'.__('Longitude', 'Geo').'</th>
			    <th scope="col">'.__('Action', 'Geo').'</th>
			  </tr>
			  </thead>
			  <tbody>
		';
	
		foreach($geolocations as $name => $coords)
		{
			list($lat, $lon) = split(',', $coords);
			$alternate = $alternate == ''? ' class="alternate"' : '';
			echo "<tr{$alternate}>";
			echo "<td>{$name}</td>";
			echo "<td><input type=\"text\" name=\"lat[{$name}]\" value=\"{$lat}\" /></td>";
			echo "<td><input type=\"text\" name=\"lon[{$name}]\" value=\"{$lon}\" /></td>";
			echo "<td><a href=\"" . add_query_arg('deleteid', $name) . "\"
				onClick=\"return confirm('" . __("Are you sure you want to delete this location entry?", 'Geo') . "');\"
				>Delete</a>
				&middot;
				<a href=\"" . get_geo_url('MapQuest', $lat, $lon) . "\">Map</a>
				</td>
			";
			echo "</tr>";
		}
	
		echo '<tr valign="bottom">
		<td><strong>'.__('New Location', 'Geo').':</strong><br/>
		<input type="text" name="new_location" value="" /></td>
		<td><input type="text" name="new_lat" value="" /></td>
		<td><input type="text" name="new_lon" value="" /></td>
		<td>&nbsp;</td>
		</tr>
		</tbody>
		</table>';
	
	
	?>
	
	<div class="submit"><input type="submit" name="Submit" value="<?php _e("Add/Update Locations", 'Geo'); ?> &raquo;" /></div>
	</form>
	
	</div>
	<?php
		if(is_callable(array('GeoMap', 'options_page'))) {
		   GeoMap::options_page();
		}
	}
	
	function prepare_database()
	/**
	* Moves WordPress 1.2 geo info to meta fields.
	*
	* Long description for the Function.
	*
	* @return	mixed	Description
	*/
	{
		global $wpdb;
		// Are we in admin?
		if(strstr($_SERVER['REQUEST_URI'], 'plugins.php'))
		{
			// Not already installed?
			if($wpdb->get_var("SELECT count(post_id) FROM {$wpdb->postmeta} WHERE meta_key = '_geo_location'") == 0)
			{
				// Put empty geo markers on any one post to mark plugin as installed
				$one_post_id = $wpdb->get_var("SELECT ID FROM {$wpdb->posts} LIMIT 1;");
				add_post_meta($one_post_id, '_geo_location', '');

				// Is there a WP 1.2-remnant geo field pair?
				$fieldrow = $wpdb->get_row("SELECT * FROM {$wpdb->posts} LIMIT 1", ARRAY_A);
				$fields = array_keys($fieldrow);
				if(in_array('post_lat', $fields))
				{
					if($latlon_posts = $wpdb->get_results("SELECT * FROM {$wpdb->posts} WHERE not isnull(post_lat) OR not isnull(post_lon);"))
					{
						foreach($latlon_posts as $post)
						{
							add_post_meta($post->ID, '_geo_location', $post->post_lat . ',' . $post->post_lon);
						}
					}
				}
				Geo::add_options();
			}
		}
		return true;
	}

	function add_options()
	/**
	* Adds the options required for geo info
	*
	* Long description for the Function.
	*
	*/
	{
		add_option('use_geo_positions', 0);
		add_option('use_default_geourl', 0);
		add_option('default_geourl_lat', 0);
		add_option('default_geourl_lon', 0);
		add_option('geo_locations', array());
	}

	function edit_form_advanced($not_used)
	/**
	* Queues the output buffer for displaying the Geo interface on the post 
	* editing form
	*
	* Long description for the Function.
	*
	* @param	$not_used	Description
	*/
	{
		ob_start(array('Geo', 'advanced_buffer'));
	}
	
	function advanced_buffer($content)
	/**
	* Displays the Geo interface on the post editing form
	*
	* Long description for the Function.
	*
	* @param	$content	Description
	*/
	{
		global $post_ID;
		
		$geolocations = get_settings('geo_locations');

		$output = '<fieldset id="postexcerpt" class="dbx-box"><h3 class="dbx-handle">' . __('Location', 'Geo') . '</h3><div class="dbx-content">';

		list($lat, $lon) = split(',', get_post_meta($post_ID, '_geo_location', true));
		$output .= '<label for="geo_lat">' . __('Latitude:', 'Geo') . ' <input size="10" type="text" value="' . $lat .'" name="geo_lat" id="geo_lat" /></label> &nbsp; ';
		$output .= '<label for="geo_lon">' . __('Longitude:', 'Geo') . ' <input size="10" type="text" value="' . $lon .'" name="geo_lon" id="geo_lon" /></label> &nbsp; ';

		$output .= '<label for="geo_select">' . __('Preset:', 'Geo') . ' <select id="geo_select" onchange="geo_choosegeo(this);"><option value="">--choose one--</option>';
		foreach($geolocations as $geolocation => $coords)
		{
			$output .= "<option value=\"$coords\"";
			if($coords == "{$lat},{$lon}") $output .= ' selected="selected"';
			$output .= ">{$geolocation}</option>\n";
		}
		$output .= '</select></label>';

		$output .= '</div></fieldset>';
		
		return preg_replace('#(<div.*?id="advancedstuff".*?'.'>)#ims', '\\1' . $output, $content, 1);
	}

	function admin_head($not_used)
	/**
	* Outputs support javascript to admin console and prepares the databse.
	*
	* Long description for the Function.
	*
	* @param	$not_used	Description
	*/
	{
		Geo::prepare_database();
		if(strstr($_SERVER['REQUEST_URI'], 'post.php'))
		{
			echo '
<script type="text/javascript">
//<![CDATA[
function geo_choosegeo(sel)
{
	var coord = sel.options[sel.selectedIndex].value;
	var ll = coord.split(",");
	var inps = document.getElementsByTagName("input");
	for(z=0;z<inps.length;z++)
	{
		if(inps[z].getAttribute("name") == "geo_lat") inps[z].setAttribute("value", ll[0]);
		if(inps[z].getAttribute("name") == "geo_lon") inps[z].setAttribute("value", ll[1]);
	}
	sel.selectedIndex = 0;
}
//]]>
</script>';
		}
	}

	function update_post($id)
	/**
	* Updates the geo info for a post when the post is updated.
	*
	* Long description for the Function.
	*
	* @param	$id	Description
	*/
	{
		delete_post_meta($id, '_geo_location');
		add_post_meta($id, '_geo_location', $_POST['geo_lat'] . ',' . $_POST['geo_lon']);
	}

	function wp_head($not_used)
	/**
	* Outputs the geo info as meta tags in the blog head.
	*
	* Long description for the Function.
	*
	* @param	$not_used	Description
	*/
	{
		global $wp_query;

		if(!get_settings('use_geo_positions')) return;

		list($lat, $lon) = split(',', get_post_meta($wp_query->post->ID, '_geo_location', true));
		if(is_single() && ($lat != '') && ($lon != ''))
		{
			$title = convert_chars(strip_tags(get_bloginfo("name")))." - ".$wp_query->post->post_title;
		}
		else if(get_settings('use_default_geourl'))
		{
			// send the default here
			$title = convert_chars(strip_tags(get_bloginfo("name")));
			$lat = get_settings('default_geourl_lat');
			$lon = get_settings('default_geourl_lon');
		}
		else
		{
			return;
		}
		echo "<meta name=\"ICBM\" content=\"{$lat}, {$lon}\" />\n";
		echo "<meta name=\"DC.title\" content=\"{$title}\" />\n";
		echo "<meta name=\"geo.position\" content=\"{$lat};{$lon}\" />\n";
	}

	function logErrors($msg)
	/**
	* Simple error logging function for debugging.
	*
	* Long description for the Function.
	*
	* @return	bool	Always true
	* @param	$msg	Description
	*/
	{
		$fp = fopen("../asy.log","a+");
		fwrite($fp, "\n\n".date("Y-m-d H:i:s - ").$msg);
		fclose($fp);
		return true;
	}

	function admin_menu($not_used)
	/**
	* Adds the Geo Info menu to the Options tab in the admin menu.
	*
	* Long description for the Function.
	*
	* @param	$not_used	Description
	*/
	{
		add_options_page(__('Geo Location Manager', 'Geo'), __('Geo Info', 'Geo'), 5, basename(__FILE__), array('Geo', 'options_page_geo'));
		//add_submenu_page('index.php', __('Geo Location Manager', 'Geo'), __('Geo Info', 'Geo'), 5, basename(__FILE__), array('Geo', 'options_page_geo'));
	}

}	// End class geo

// Note the array as the second parameter for calling static methods on our nice, safe object namespace
// For instance, array('Geo', 'edit_form_advanced') calls Geo::edit_form_advanced()
add_action('edit_form_advanced', array('Geo', 'edit_form_advanced'));
add_action('admin_head', array('Geo', 'admin_head'));
add_action('wp_head', array('Geo', 'wp_head'));
add_action('save_post', array('Geo', 'update_post'));
//add_action('edit_post', array('Geo', 'update_post'));
//add_action('publish_post', array('Geo', 'update_post'));
add_action('admin_menu', array('Geo', 'admin_menu'));

function pingGeoURL($blog_ID)
/**
* Pings geourl.org with blog page ID.
*
* Long description for the Function.
*
* @param	$blog_ID	Description
*/
{
	$ourUrl = get_settings('home') ."/index.php?p=".$blog_ID;
	$host="geourl.org";
	$path="/ping/?p=".$ourUrl;
	getRemoteFile($host,$path);
}

function get_Lat()
/**
* Returns the latitude setting for a post.
*
* Long description for the Function.
*
* @return	string	Description
*/
{
	global $post;

	list($lat, $lon) = split(',', get_post_meta($post->ID, '_geo_location', true));
	if ($lat != '') {
		return trim($lat);
	} else if(get_settings('use_default_geourl')) {
		return trim(get_settings('default_geourl_lat'));
	}

	return '';
}

function get_Lon()
/**
* Returns the longitude setting for a post.
*
* Long description for the Function.
*
* @return	string	Description
*/
{
	global $post;

	list($lat, $lon) = split(',', get_post_meta($post->ID, '_geo_location', true));
	if ($lon != '') {
		return trim($lon);
	} else if(get_settings('use_default_geourl')) {
		return trim(get_settings('default_geourl_lon'));
	}

	 return '';
}

function the_Lat()
/**
* Outputs the latitude for a post.
*
* Long description for the Function.
*
* @return	string	Description
*/
{
	if(get_settings('use_geo_positions')) {
		if(get_Lat() > 0) {
			echo "".get_Lat()."N";
		} else {
			echo "".get_Lat()."S";
		}
	}
}

function the_Lon()
/**
* Outputs the longitude for a post.
*
* Long description for the Function.
*
* @return	string	Description
*/
{
	global $id, $postdata;
	if(get_settings('use_geo_positions')) {
		if(get_Lon() < 0) {
			$temp = get_Lon() * -1;
			echo "".$temp."W";
		} else {
			echo "".get_Lon()."E";
		}
	}
}

function geo_PopUpScript()
/**
* Outputs the popup javascript used by the dropdown popup-er.
*
* Long description for the Function.
*
* @return	string	Description
*/
{
	echo "
		<script type='text/javascript'>
		function formHandler(form) {
			var URL = form.site.options[form.site.selectedIndex].value;
			if(URL != \".\") {
				popup = window.open(URL,\"MenuPopup\");
			}
		}
		</script>";
}

function geo_urls($lat = '', $lon = '')
/**
* Returns an array of named URLs for the llat and lon specified.
*
* Long description for the Function.
*
* @return	array	Description
*/
{
	$lat = $lat == ''? get_Lat() : $lat;
	$lon = $lon == ''? get_Lon() : $lon;
	$title = get_place_name($lat, $lon);
	$ary = array (
		'AcmeMap' => array("http://www.acme.com/mapper?lat={$lat}&amp;long={$lon}&amp;scale=11&amp;theme=Image&amp;width=3&amp;height=2&amp;dot=Yes", __('Acme Mapper', 'Geo')),
		'GeoURL' => array("http://geourl.org/near/?lat={$lat}&amp;lon={$lon}&amp;dist=500", __('GeoURLs near here', 'Geo')),
		'GeoCache' => array("http://www.geocaching.com/seek/nearest.aspx?origin_lat={$lat}&amp;origin_long={$lon}&amp;dist=5", __('Geocaches near here', 'Geo')),
		'MapQuest' => array("http://www.mapquest.com/maps/map.adp?latlongtype=decimal&amp;latitude={$lat}&amp;longitude={$lon}", __('Mapquest map of this spot', 'Geo')),
		'SideBit' => array("http://www.sidebit.com/ProjectGeoURLMap.php?lat={$lat}&amp;lon={$lon}", __('SideBit URL Map of this spot', 'Geo')),
		'DegreeConfluence' => array("http://confluence.org/confluence.php?lat={$lat}&amp;lon={$lon}", __('Confluence.org near here', 'Geo')),
		'TopoZone' => array("http://www.topozone.com/map.asp?lat={$lat}&amp;lon={$lon}", __('Topozone near here', 'Geo')),
		'FindU' => array("http://www.findu.com/cgi-bin/near.cgi?lat={$lat}&amp;lon={$lon}&amp;scale=100000&amp;zoom=50&amp;type=1&amp;icon=0&amp;&amp;scriptfile=http://mapserver.maptech.com/api/espn/index.cfm", __('FindU near here', 'Geo')),
		'MapTech' => array("http://mapserver.maptech.com/api/espn/index.cfm?lat={$lat}&amp;lon={$lon}", __('Maptech near here', 'Geo')),
		//'GoogleMaps' => array("http://maps.google.com/maps?ll={$lat}%2C{$lon}&amp;spn=0.015839,0.032747&amp;hl=en", __('Google Maps', 'Geo')),
		'GoogleMaps' => array("http://maps.google.com/maps?q={$lat}+{$lon}+({$title})&amp;hl=en", __('Google Maps', 'Geo')),
	);
	return $ary;
}

function geo_UrlPopNav()
/**
* Creates a dropdown popup-er for viewing geo sites for a post's lat/long.
*
* Long description for the Function.
*
*/
{
	$sites = geo_urls();

	echo '<form action=""><div>\n<select name="site" size="1" onchange="formHandler(this.form);" >'."\n";
	echo '<option value=".">' . sprintf(__("Sites referencing %s x %s", 'Geo'), get_Lat(), get_Lon()) . "</option>\n";
	foreach($sites as $site) {
			echo "\t".'<option value="'.$site[0].'">'.$site[1]."</option>\n";
	}
	echo '</select>\n</div></form>'."\n";
}

function longitude_invalid()
/**
* Validates the longitude value of the current post.
*
* Long description for the Function.
*
* @return	boolean	True if longitude is out of range.
*/
{
	if (get_Lon() == null) return true;
	if (get_Lon() > 360) return true;
	if (get_Lon() < -360) return true;
}

function get_geo_url($index, $lat = '', $lon = '')
/**
* Returns a URL for the indexed geo site at the given coordinates.
*
* Long description for the Function.
*
* @param	$index	The associative index of the URL array to return.
* @param	$lat	Optional latitude for the URL, if not specified then the current is used.
* @param	$lon	Optional longitude for the URL, if not specified then the current is used.
*
* @return	string	The URL of the requested site.
*/
{
	$lat = $lat == ''? get_Lat() : $lat;
	$lon = $lon == ''? get_Lon() : $lon;
	$urls = geo_urls($lat, $lon);
	return $urls[$index][0];
}

function the_AcmeMap_Url()
/**
* Outputs a URL to the AcmeMap site for the current geo coordinates.
*
* Long description for the Function.
*
*/
{
	/*
	// Shouldn't this function work if a coordinate is set?
	// Can't we correct an invalid longitude?
	if (!get_settings('use_geo_positions')) return;
	if (longitude_invalid()) return;
	*/
	echo get_geo_url('AcmeMap');
}

function the_GeoURL_Url()
/**
* Outputs a URL to the GeoURL site for the current geo coordinates.
*
* Long description for the Function.
*
*/
{
	echo get_geo_url('GeoURL');
}

function the_GeoCache_Url()
/**
* Outputs a URL to the GeoCaching site for the current geo coordinates.
*
* Long description for the Function.
*
*/
{
	echo get_geo_url('GeoCache');
}

function the_MapQuest_Url()
/**
* Outputs a URL to the MapQuest site for the current geo coordinates.
*
* Long description for the Function.
*
*/
{
	echo get_geo_url('MapQuest');
}

function the_SideBit_Url()
/**
* Outputs a URL to the SideBit site for the current geo coordinates.
*
* Long description for the Function.
*
*/
{
	echo get_geo_url('SideBit');
}

function the_DegreeConfluence_Url()
/**
* Outputs a URL to the DegreeConfluence site for the current geo coordinates.
*
* Long description for the Function.
*
*/
{
	echo get_geo_url('DegreeConfluence');
}

function the_TopoZone_Url()
/**
* Outputs a URL to the TopoZone site for the current geo coordinates.
*
* Long description for the Function.
*
*/
{
	echo get_geo_url('TopoZone');
}

function the_FindU_Url()
/**
* Outputs a URL to the FindU site for the current geo coordinates.
*
* Long description for the Function.
*
*/
{
	echo get_geo_url('FindU');
}

function the_MapTech_Url()
/**
* Outputs a URL to the MapTech site for the current geo coordinates.
*
* Long description for the Function.
*
*/
{
	echo get_geo_url('MapTech');
}

function the_GoogleMap_Url() {
/**
* Outputs a URL to Google Maps for the current geo coordinates.
*
* Long description for the Function.
*
*/
	echo get_geo_url('GoogleMaps');
}


function geo_distance($lat1, $lon1, $lat2, $lon2, $unit = 'm' )
/**
* Returns the distance between two geo coordinates.
*
* Long description for the Function.
*
* @param	$lat1	The latitude of the first coordinate
* @param	$lon1	The longitude of the first coordinate
* @param	$lat2	The latitude of the second coordinate
* @param	$lon2	The longitude of the second coordinate
* @param	$unit	'm' for miles, 'k' for kilometers, 'n' for nautical miles.
*
* @return	float	The distance between the to coordinates.
*/
{
	$theta = $lon1 - $lon2;
	$dist = sin(deg2rad($lat1)) * sin(deg2rad($lat2)) +  cos(deg2rad($lat1)) * cos(deg2rad($lat2)) * cos(deg2rad($theta));
	$dist = acos($dist);
	$dist = rad2deg($dist);
	$miles = $dist * 60 * 1.1515;
	$unit = strtoupper($unit);

	if ($unit == "k") {
		return ($miles * 1.609344);
	} else if ($unit == "n") {
			return ($miles * 0.8684);
		} else {
				return $miles;
			}
}

function pretty_degrees($decimal_degrees)
{
	$deg = floor($decimal_degrees);
	$decimal_degrees = ($decimal_degrees - $deg) * 60;
	$min = floor($decimal_degrees);
	$decimal_degrees = ($decimal_degrees - $min) * 60;
	$sec = $decimal_degrees;
	return "{$deg}&deg; {$min}' {$sec}\"";
}

function get_place_name($lat = 'x', $lon = 'x')
{
	global $post;
	
	if($lat=='x' || $lon=='x') {
		list($lat, $lon) = split(',', get_post_meta($post->ID, '_geo_location', true));
		if ($lat == '') {
			$lat = get_settings('default_geourl_lat');
			$lon = get_settings('default_geourl_lon');
		}
	}
	$geolocations = get_settings('geo_locations');

	foreach($geolocations as $geolocation => $coords)
	{
		if($coords == "{$lat},{$lon}") return $geolocation;
	}
	return '';
}

function the_location($format = '<a href="%4$s">%1$s</a> at %2$s, %3$s', $defaultlocation = 'Unnamed location', $service='GoogleMaps')
{
	global $post;

	list($lat, $lon) = split(',', get_post_meta($post->ID, '_geo_location', true));
	if ($lat == '') {
		$lat = get_settings('default_geourl_lat');
		$lon = get_settings('default_geourl_lon');
	}

	$place = get_place_name($lat, $lon);
	if($place == '') $place = $defaultlocation;
	
	if($lat > 0) {
		$outlat = pretty_degrees($lat)."N";
	} else {
		$outlat = pretty_degrees($lat)."S";
	}
	if($lon < 0) {
		$temp = get_Lon() * -1;
		$outlon = pretty_degrees($temp)."W";
	} else {
		$outlon = pretty_degrees($lon)."E";
	}
	
	echo sprintf($format, $place, $outlat, $outlon, get_geo_url($service, $lat, $lon));
}

function get_the_rss_geotags()
/**
* Returns the tags with geo information for inclusion in RSS 2.0 feeds.
*
* Include a call to this function inside the <item> tag to add tags with geo information if available.
* To keep the feed valid add these namespace declarations to the <rss> tag:
* xmlns:geo="http://www.w3.org/2003/01/geo/wgs84_pos#"
* xmlns:icbm="http://postneo.com/icbm"
* xmlns:geourl="http://geourl.org/rss/module/"
*/
{
	global $wp_query;

	if(!get_settings('use_geo_positions')) return;

	list($lat, $lon) = split(',', get_post_meta($wp_query->post->ID, '_geo_location', true));
	if ($lat == '' || $lon == '') {
		if (get_settings('use_default_geourl')){
			// send the default here
			$lat = get_settings('default_geourl_lat');
			$lon = get_settings('default_geourl_lon');
		}
	}
	if ($lat != '' && $lon != '') {
		echo "<geo:lat>$lat</geo:lat><geo:long>$lon</geo:long>\n"
				. "<icbm:latitude>$lat</icbm:latitude><icbm:longitude>$lon</icbm:longitude>\n"
				. "<geourl:longitude>$lon</geourl:longitude><geourl:latitude>$lat</geourl:latitude>\n";
	}
}

?>
